<?php

	define('APS_DEVELOPMENT_MODE', true);
	require "aps/2/runtime.php";	

	/**
	* Class context
	* @type("http://aps-standard.org/samples/counters/context/1.0")
	* @implements("http://aps-standard.org/types/core/subscription/service/1.0")
	*/
	
	class context extends \APS\ResourceBase
	{
	   ## Strong relation (link) to the application instance
	   /**
	   * @link("http://aps-standard.org/samples/counters/cloud/1.0")
	   * @required
	   */
	   public $cloud;
	
	   ## Weak relation (link) to collection of VPSes
	   /**
	    * @link("http://aps-standard.org/samples/counters/vps/1.0[]")
	    */
	   public $vpses;
	   
	   /**
	    * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
	    * @description("Total disk space usage, GB")
	    * @unit("gb")
	    */
	   public $diskUsageTotal;

	   /**
	    * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
	    * @description("Total CPU usage, CPUcore-hour")
	    * @unit("item-h")
	    */
	   public $cpuUsageTotal;
	   
	   /**
	   * @type("integer")
	   * @description("Accumulates CPU usage in the period between retrieve operations")
	    * @unit("item-h")
	    */
	   public $cpuAccumulator;
	   
	   /**
	    * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
	    * @description("Total memory usage, MB-hour")
	    * @unit("mb-h")
	    */
	   public $memoryUsageTotal;

	   /**
	   * @type("integer")
	   * @description("Accumulates memory usage in the period between retrieve operations")
	    * @unit("mb-h")
	    */
	   public $memoryAccumulator;

	   /**
	    * @type("integer")
	    * @description("Time stamp of the last resource usage counting")
	    */
	   public $time1;
	   
	   public function provision() {
	      ## Reset counters and record the init time
	   	$this->time1 = time();
	   	$this->cpuUsageTotal->usage = 0;
	   	$this->memoryUsageTotal->usage = 0;
	   	$this->diskUsageTotal->usage = 0;

         ## Reset auxiliary accumulators
	   	$this->cpuAccumulator = 0;
	   	$this->memoryAccumulator = 0;
	   }
	   
	   public function retrieve() {
	   	
	   	## Initialize the local variables
	   	$currentTime = time();
	   	$timeDelta = ($currentTime - $this->time1) / 3600;      // Elapsed time in hours
	   	$this->time1 = $currentTime;
	   	
	   	$cpuRate = 0;
	   	$ramRate = 0;
	   	$cpuUsage = $this->cpuAccumulator;
	   	$ramUsage = $this->memoryAccumulator;
	   	$diskUsage = 0;

	   	## Reset the accumulators
	   	$this->cpuAccumulator = 0;
	   	$this->memoryAccumulator = 0;
	   	$apsc = \APS\Request::getController();
	   	$apsc->updateResource($this);

	   	## Collect resource usage from all VPSes
	   	foreach ($this->vpses as $vps) {
	   		if ($vps->state == 'Running') {
	   			$cpuRate += $vps->hardware->CPU->number;
	   			$ramRate += $vps->hardware->memory;
	   		}
	   		$diskUsage += $vps->hardware->diskspace;
	   	}
	   
	   	## Delta during the period between the retrieve polls
	   	$cpuUsage += floor($cpuRate * $timeDelta);
	   	$ramUsage += floor($ramRate * $timeDelta);

	   	## Update the APS resource counters
	   	$this->cpuUsageTotal->usage += $cpuUsage;
	   	$this->memoryUsageTotal->usage += $ramUsage;
	   	$this->diskUsageTotal->usage = $diskUsage;

	   }

	   /**
	    * @verb(GET)
	    * @path("/resourceCounting")
	    */
	   public function resourceCounting() {
	   	$currentTime = time();
	   	$timeDelta = ($currentTime - $this->time1) / 3600; // Elapsed time in hours
	   	$this->time1 = $currentTime;
	   
	   	$cpuRate = 0;
	   	$ramRate = 0;
	   	foreach ($this->vpses as $vps) {
	   		if ($vps->state == 'Running') {
	   			$cpuRate += $vps->hardware->CPU->number;
	   			$ramRate += $vps->hardware->memory;
	   		}
	   	}
	   	$this->cpuAccumulator += floor($cpuRate * $timeDelta);  // Additive
	   	$this->memoryAccumulator += floor($ramRate * $timeDelta);  // Additive
	   	$apsc = \APS\Request::getController();
	   	$apsc->updateResource($this);
	   }
	}
?>
