define([
  "dojo/_base/declare",
  "dojo/when",
  "dojox/mvc/getStateful",
  "dojox/mvc/at",
  "dojo/_base/array",
  
  "aps/View",
  "aps/ResourceStore",
  "aps/xhr",
  "./displayError"
], function (declare, when, getStateful, at, arr, View, Store, xhr, displayError) {
	var page, grid, self, contextId; // Global variable to exchange data between methods
	return declare(View, {
		init: function() {
			self = this;   // Refer to the declared View object
			/* Define the data store */
			var vpsStore = new Store({
				apsType: "http://aps-standard.org/samples/counters/vps/1.0",
				target: "/aps/2/resources/"
			});

			/* Initialize a model to collect resource usage */
			   self.usageModel = getStateful({
			   "cpu": { "limit": 0, "usage": 0 },
			   "ram": { "limit": 0, "usage": 0 },
			   "disk": { "limit": 0, "usage": 0 }
			   });
			   
			/* Define widget processing logic  */
			var add = function() {
				/* Start the process of creating a VPS by going to the first screen */
				aps.apsc.gotoView("vps-wizard");
			};
			var refresh = function() {
				   xhr(
				      "/aps/2/resources/" + contextId + "/resourceCounting",
				      { method: "GET",  handleAs: "json" }
				   ).then(function() {
				      return xhr(
				         "/aps/2/resources/" + contextId,
				         { method: "GET",  handleAs: "json" }
				      );
				   }).then(function(mgmtContext) {
				      self.usageModel.set("cpu", mgmtContext.cpuUsageTotal);
				      self.usageModel.set("ram", mgmtContext.memoryUsageTotal);
				      self.usageModel.set("disk", mgmtContext.diskUsageTotal);
				      grid.refresh();
				   });
			};

			function changeState(state, btn) {
			   var sel = grid.get("selectionArray"),
			   counter = sel.length;

			   /* Clear the current messages on the screen */
			   page.get("messageList").removeAll();

			   arr.forEach(sel, function(vpsId) {
			      console.log("I'm trying to change state of VPS with id = [" + vpsId + "]");

			      /* Save the VPS state property */
			      when(
			         xhr(
			            "/aps/2/resources/" + vpsId + "/" + state,
			            { method: "GET",  handleAs: "text" }
			         ),
			          /* If success, process the next VPS until the list is empty */
			         function() {
			            console.log("State of VPS with id = [" + vpsId + "] changed");
			            sel.splice(sel.indexOf(vpsId),1);
			            grid.refresh();
			            if (--counter === 0) { btn.cancel(); } /* Remove busy state for button */
			         },
			         /* If failure, call the error handler */
			         function(e) {
			            displayError(e);
			            if (--counter === 0) { btn.cancel(); }
			         }
			      ); // End of when();
			   });
			}
			
			var stop = function() {
			      changeState("stop", this);
			};
			var start = function() {
			      changeState("start", this);
			};
			var remove = function() {
			    var btn = this;
			    /* Get confirmation from the user for the delete operation */
			    if (!confirm("Are you sure you want delete VPSes?")) {
			       btn.cancel();
			       return;
			    }

			    var sel = self.grid.get("selectionArray");
			    var counter = sel.length;

			    /* Clear the current messages on the screen */
			    self.page.get("messageList").removeAll();

			    sel.forEach(function(vpsId){
			       console.log("I'm trying to delete VPS with id = [" + vpsId + "]");

			       /* Remove the VPS from the APS controller DB */
			       when(vpsStore.remove(vpsId),
			           /* If success, process the next VPS until the list is empty */
			          function(){
			             console.log("VPS with id = [" + vpsId + "] removed");
			             /* Remove the processed VPS from the selection */
			             sel.splice(sel.indexOf(vpsId),1);
			             self.grid.refresh();
			             if (--counter === 0) { btn.cancel(); }
			          },
			          /* If failure, call the error handler */
			          function(e){
			             displayError(e);
			             if (--counter === 0) { btn.cancel(); }
			          }
			       );
			    });
			};
			
			/* Define and return widgets */
			return [
				["aps/Grid", {
					id:                this.genId("srv_grid"),
					store:             vpsStore,
					selectionMode:     "multiple",
					apsResourceViewId: "server-edit",
					noDataText: "No servers provisioned",
					noEntriesFoundText: "No servers meet your search criteria",
					columns: [{
			                     field: "name",
			                     name: "Name",
			                     type: "resourceName",
			                     filter: true
			             }, {
			                     field: "hardware.memory",
			                     name: "RAM",
			                     title: "RAM",
			                     type: "integer",
			                     filter: true
			             }, {
			                     field: "hardware.diskspace",
			                     name: "Disk Space",
			                     title: "Disk",
			                     type: "integer",
			                     filter: true
			             }, {
			                     field: "hardware.CPU.number",
			                     name: "CPU",
			                     type: "integer",
			                     filter: true
			             }, {
			                     field: "platform.OS.name",
			                     name: "OS",
			                     filter: true
			             }, {
			                     field: "state",
			                     name: "State",
			                     filter: true
			             }]}, [

			         ["aps/Toolbar", [
			               ["aps/ToolbarButton", {
			                    id: this.genId("srv_new"),
			                    iconClass:"fa-plus",
			                    type: "primary",
			                    label: "New",
			                    onClick: add
			               }],
			               ["aps/ToolbarButton", {
			                    id: this.genId("srv_start"),
			                    iconClass:"fa-play",
			                    type: "success",
			                    label: "Start",
			                    requireItems: true,
			                    onClick: start
			               }],
			               ["aps/ToolbarButton", {
			                    id: this.genId("srv_stop"),
			                    iconClass:"fa-pause",
			                    type: "warning",
			                    label: "Stop",
			                    requireItems: true,
			                    onClick: stop
			               }],
			               ["aps/ToolbarButton", {
			                    id: "srv_delete",
			                    iconClass:"fa-trash",
			                    type: "danger",
			                    label: "Delete",
			                    requireItems: true,
			                    onClick: remove
			               }],
			               ["aps/ToolbarButton", {
			                    id: this.genId("srv_refresh"),
			                    iconClass:"fa-refresh",
			                    label: "Refresh",
			                    autoBusy: false,
			                    onClick: refresh
			               }]
			         ]]
				]],
				["aps/Tiles", {
						title: "Resource consumption"
					},[
						["aps/Tile", {
								title: "CPU power",
								gridSize: "md-4"
							}, [
								["aps/UsageInfo", {
									showPie: false,
									value: at(self.usageModel.cpu, "usage"),
									maximum: at(self.usageModel.cpu, "limit"),
									textFirstNumber: '${value}',
									description: 'core-hours',
									hint: 'Limit: ${maximum}'
						}]]],
						["aps/Tile", {
								title: "Memory usage",
								gridSize: "md-4"
							}, [
								["aps/UsageInfo", {
									showPie: false,
									value: at(self.usageModel.ram, "usage"),
									maximum: at(self.usageModel.ram, "limit"),
									textFirstNumber: '${value}',
									description: 'MB-hours',
									hint: 'Limit: ${maximum}'
						}]]],
						["aps/Tile", {
								title: "Disk usage",
								gridSize: "md-4"
							}, [
								["aps/UsageInfo", {
									textSuffix: "GB",
									value: at(self.usageModel.disk, "usage"),
									maximum: at(self.usageModel.disk, "limit"),
									textFirstNumber: '${value}',
									textSecondNumber: '${maximum}'
					}]]]
				]]
			];
		},

		onContext: function(context) {
		   grid = this.byId("srv_grid");
		   page = this.byId("apsPageContainer");
		   var mgmtContext = context.vars.context;
		   contextId = mgmtContext.aps.id;
		   self.usageModel.set("cpu", mgmtContext.cpuUsageTotal);
		   self.usageModel.set("ram", mgmtContext.memoryUsageTotal);
		   self.usageModel.set("disk", mgmtContext.diskUsageTotal);
		   grid.refresh();
		   aps.apsc.hideLoading();
		},
			
		onHide: function() {
			this.byId("srv_new").cancel();
		}
});
});
